@@ -10,13 +10,15 @@ use strict;
 
 use Test::Smoke;
 use vars qw($VERSION);
-$VERSION = Test::Smoke->VERSION; # $Id: mkovz.pl 155 2003-06-06 15:42:50Z abeltje $
+$VERSION = Test::Smoke->VERSION; 
+# $Id: mkovz.pl 251 2003-07-20 15:25:01Z abeltje $
 
 use File::Spec;
 use Cwd;
 use Text::ParseWords;
 
 use FindBin;
+use lib $FindBin::Bin;
 use lib File::Spec->catdir( $FindBin::Bin, 'lib' );
 use Test::Smoke::Util qw( get_ncpu get_smoked_Config );
 
@@ -24,6 +26,7 @@ my $email = shift || 'daily-build-reports@perl.org';
 my $testd = shift || cwd ();
 my $mailer = "/usr/bin/mailx";
 my $locale = shift;
+my $defaultenv = shift;
 
 my %Config = get_smoked_Config( $testd, qw( version cf_email
                                             osname osvers 
@@ -50,6 +53,7 @@ $Config{archname} .= "/$cpus" if $cpus;
 
 my $p_version = sprintf "%d.%03d%03d", split /\./, $Config{version};
 my $is56x = $p_version >= 5.006 && $p_version < 5.007;
+$defaultenv ||= $is56x;
 
 =head1 NAME
 
@@ -95,7 +99,7 @@ It's a hack! It should be picked up from B<mktest.out>
 
 =cut
 
-my @layers = $is56x ? qw( stdio ) : qw( stdio perlio );
+my @layers = $defaultenv ? qw( stdio ) : qw( stdio perlio );
 $locale and push @layers, "locale";
 
 my (%rpt, @confs, %confs, @manifest, $common_cfg);
@@ -124,6 +128,7 @@ select RPT;
 my $perlio = "";
 my $conf   = "";
 my $debug  = "";
+my %times  = ( start => undef, total => 0 );
 $rpt{patch} = "?";
 my ($out, @out) = (File::Spec->catfile ($testd, "mktest.out"), 1 .. 5);
 open OUT, "<$out" or die "Can't open $out: $!";
@@ -136,6 +141,14 @@ for (<OUT>) {
     unshift @out, $_;
     chomp $out[0];
 
+    if ( my( $status, $time ) = /(Started|Stopped) smoke at (\d+)/ ) {
+        if ( $status eq "Started" ) {
+            $times{start} = $time;
+        } else {
+            $times{total} += ($time - $times{start}) if defined $times{start};
+        }
+        next;
+    }
     if (m/^\s*Smoking patch (\d+)/) {
         $rpt{patch} = $1;
         next;
@@ -161,7 +174,11 @@ for (<OUT>) {
     }
     if (m/PERLIO\s*=\s*(\w+)/) {
         $perlio = $1;
-#        next;
+        if ( $^O =~  /MSWin32/ ) {
+            s/^PERLIO\s*=\s+\w+io(?: :crlf)?\s*//;
+        } else {
+            next;
+        }
     }
     if (m/^\s*All tests successful/) {
         $rpt{$conf}{$debug}{$perlio} = "O";
@@ -220,11 +237,13 @@ $Config{ccvers}	||= $Config{gccversion} || $Config{ccversion};
 require Sys::Hostname;
 my $this_host = Sys::Hostname::hostname() || "nohost";
 my $this_pver = $^V ? sprintf "%vd", $^V : $];
+$times{showtime} = time_in_hhmm( $times{total} );
+my $time_msg  = $times{showtime} ? " [$times{showtime}]" : "";
 
 print <<EOH;
 Automated smoke report for $Config{version} patch $rpt{patch} on $Config{osname} - $Config{osvers} ($Config{archname})
 ($this_host) using $Config{cc} version $Config{ccvers}
-Report generated by Test::Smoke v$VERSION (perl $this_pver)
+Report by Test::Smoke v$VERSION (perl $this_pver)$time_msg
 
 O = OK  F = Failure(s), extended report at the bottom
 ? = still running or test results not (yet) available
@@ -295,7 +314,7 @@ if ( @rpt_sum_stat ) {
     $rpt_summary = $count{o} == 0 ? 'PASS' : 'PASS-so-far';
 }
 
-print $locale ? <<EOL : $is56x ? <<EOS : <<EOE;
+print $locale ? <<EOL : $defaultenv ? <<EOS : <<EOE;
 | | | | | +- LC_ALL = $locale -DDEBUGGING
 | | | | +--- PERLIO = perlio -DDEBUGGING
 | | | +----- PERLIO = stdio  -DDEBUGGING
@@ -387,6 +406,27 @@ sub win32_username
     return Win32::LoginName() || 'unknown';
 } # win32_username
 
+sub time_in_hhmm {
+    my $diff = shift;
+
+    my $digits = $diff =~ /\./ ? 3 : 0;
+    my $days = int( $diff / (24*60*60) );
+    $diff -= 24*60*60 * $days;
+    my $hour = int( $diff / (60*60) );
+    $diff -= 60*60 * $hour;
+    my $mins = int( $diff / 60 );
+    $diff -=  60 * $mins;
+
+    my @parts;
+    $days and push @parts, sprintf "%d day%s",   $days, $days == 1 ? "" : 's';
+    $hour and push @parts, sprintf "%d hour%s",  $hour, $hour == 1 ? "" : 's';
+    $mins and push @parts, sprintf "%d minute%s",$mins, $mins == 1 ? "" : 's';
+    $diff && !$days && !$hour and
+        push @parts, sprintf "%.${digits}f seconds", $diff;
+
+    return join " ", @parts;
+}
+
 =head1 CHANGES
 
 For more recent changes see B<ChangeLog>.
